Man page: Move the description of --info=progress2 to a better place.
[rsync/rsync.git] / cleanup.c
index e59565d..cf5f212 100644 (file)
--- a/cleanup.c
+++ b/cleanup.c
@@ -4,7 +4,7 @@
  * Copyright (C) 1996-2000 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2002 Martin Pool
- * Copyright (C) 2003-2008 Wayne Davison
+ * Copyright (C) 2003-2009 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
 
 extern int am_server;
 extern int am_daemon;
+extern int am_receiver;
 extern int io_error;
 extern int keep_partial;
 extern int got_xfer_error;
+extern int protocol_version;
+extern int output_needs_newline;
 extern char *partial_dir;
 extern char *logfile_name;
 
+BOOL shutting_down = False;
+
 #ifdef HAVE_SIGACTION
 static struct sigaction sigact;
 #endif
@@ -93,36 +98,49 @@ pid_t cleanup_child_pid = -1;
  **/
 NORETURN void _exit_cleanup(int code, const char *file, int line)
 {
-       static int cleanup_step = 0;
-       static int exit_code = 0;
+       static int switch_step = 0;
+       static int exit_code = 0, exit_line = 0;
+       static const char *exit_file = NULL;
        static int unmodified_code = 0;
 
        SIGACTION(SIGUSR1, SIG_IGN);
        SIGACTION(SIGUSR2, SIG_IGN);
 
-       if (exit_code) /* Preserve first error code when recursing. */
+       if (exit_code) { /* Preserve first exit info when recursing. */
                code = exit_code;
+               file = exit_file;
+               line = exit_line;
+       }
 
        /* If this is the exit at the end of the run, the server side
-        * should not attempt to output a message (see log.c). */
+        * should not attempt to output a message (see log_exit()). */
        if (am_server && code == 0)
                am_server = 2;
 
        /* Some of our actions might cause a recursive call back here, so we
         * keep track of where we are in the cleanup and never repeat a step. */
-       switch (cleanup_step) {
-#include "case_N.h" /* case 0: cleanup_step++; */
+       switch (switch_step) {
+#include "case_N.h" /* case 0: */
+               switch_step++;
 
                exit_code = unmodified_code = code;
+               exit_file = file;
+               exit_line = line;
 
-               if (verbose > 3) {
+               if (output_needs_newline) {
+                       fputc('\n', stdout);
+                       output_needs_newline = 0;
+               }
+
+               if (DEBUG_GTE(EXIT, 2)) {
                        rprintf(FINFO,
-                               "_exit_cleanup(code=%d, file=%s, line=%d): entered\n",
-                               code, file, line);
+                               "[%s] _exit_cleanup(code=%d, file=%s, line=%d): entered\n",
+                               who_am_i(), code, file, line);
                }
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
                if (cleanup_child_pid != -1) {
                        int status;
@@ -136,6 +154,7 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
                if (cleanup_got_literal && cleanup_fname && cleanup_new_fname
                 && keep_partial && handle_partial_dir(cleanup_new_fname, PDIR_CREATE)) {
@@ -153,11 +172,14 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
-               io_flush(FULL_FLUSH);
+               if (!code || am_server || am_receiver)
+                       io_flush(FULL_FLUSH);
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
                if (cleanup_fname)
                        do_unlink(cleanup_fname);
@@ -178,21 +200,42 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
                                code = exit_code = RERR_PARTIAL;
                }
 
-               if (code || am_daemon || (logfile_name && (am_server || !verbose)))
+               /* If line < 0, this exit is after a MSG_ERROR_EXIT event, so
+                * we don't want to output a duplicate error. */
+               if ((code && line > 0)
+                || am_daemon || (logfile_name && (am_server || !INFO_GTE(STATS, 1))))
                        log_exit(code, file, line);
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
-               if (verbose > 2) {
+               if (DEBUG_GTE(EXIT, 1)) {
                        rprintf(FINFO,
-                               "_exit_cleanup(code=%d, file=%s, line=%d): "
+                               "[%s] _exit_cleanup(code=%d, file=%s, line=%d): "
                                "about to call exit(%d)\n",
-                               unmodified_code, file, line, code);
+                               who_am_i(), unmodified_code, file, line, code);
+               }
+
+               /* FALLTHROUGH */
+#include "case_N.h"
+               switch_step++;
+
+               if (exit_code && exit_code != RERR_SOCKETIO && exit_code != RERR_STREAMIO && exit_code != RERR_SIGNAL1
+                && exit_code != RERR_TIMEOUT && !shutting_down && (protocol_version >= 31 || am_receiver)) {
+                       if (line > 0) {
+                               if (DEBUG_GTE(EXIT, 3)) {
+                                       rprintf(FINFO, "[%s] sending MSG_ERROR_EXIT with exit_code %d\n",
+                                               who_am_i(), exit_code);
+                               }
+                               send_msg_int(MSG_ERROR_EXIT, exit_code);
+                       }
+                       noop_io_until_death();
                }
 
                /* FALLTHROUGH */
 #include "case_N.h"
+               switch_step++;
 
                if (am_server && code)
                        msleep(100);